@@ -226,111 +226,56 @@ inline void default_add_entity(Event<TSeq> & a, Model<TSeq> *)
 
         if (p->get_n_entities() > e->size()) // Slower search through the agent
         {
-            for (size_t i = 0u; i < e->size(); ++i)
-                if(static_cast<int>(e->operator[](i)) == p->get_id())
+            for (const Agent<TSeq> & agent: e->get_agents())
+                if(agent.get_id() == p->get_id())
                     throw std::logic_error("An entity cannot be reassigned to an agent.");
         }
         else                                 // Slower search through the entity
         {
-            for (size_t i = 0u; i < p->get_n_entities(); ++i)
-                if(p->get_entity(i).get_id() == e->get_id())
+            for (const Entity<TSeq> & entity: p->get_entities())
+                if(entity.get_id() == e->get_id())
                     throw std::logic_error("An entity cannot be reassigned to an agent.");
         }
 
         // It means that agent and entity were not associated.
     }
 
-    // Adding the entity to the agent
-    if (++p->n_entities <= p->entities.size())
-    {
-
-        p->entities[p->n_entities - 1]           = e->get_id();
-        p->entities_locations[p->n_entities - 1] = e->n_agents;
-
-    } else
-    {
-        p->entities.push_back(e->get_id());
-        p->entities_locations.push_back(e->n_agents);
-    }
-
-    // Adding the agent to the entity
-    // Adding the entity to the agent
-    if (++e->n_agents <= e->agents.size())
-    {
-
-        e->agents[e->n_agents - 1]          = p->get_id();
-        // Adjusted by '-1' since the list of entities in the agent just grew.
-        e->agents_location[e->n_agents - 1] = p->n_entities - 1;
-
-    } else
-    {
-        e->agents.push_back(p->get_id());
-        e->agents_location.push_back(p->n_entities - 1);
-    }
-
-    // Today was the last modification
-    // e->date_last_add_or_remove = m->today();
+    // Adding the to agent and the entity
+    p->entities.push_back(std::ref(*e));
+    e->agents.push_back(std::ref(*p));
     
 }
 
 template<typename TSeq>
-inline void default_rm_entity(Event<TSeq> & a, Model<TSeq> * m)
+inline void default_rm_entity(Event<TSeq> & a, Model<TSeq> *)
 {
     
-    Agent<TSeq> *  p = a.agent;    
-    Entity<TSeq> * e = a.entity;
-    size_t idx_agent_in_entity = a.idx_agent;
-    size_t idx_entity_in_agent = a.idx_object;
+    Agent<TSeq> &  p = *a.agent;    
+    Entity<TSeq> & e = *a.entity;
+    
+    // Remove entity from agent's entity list
+    p.entities.erase(
+        std::remove_if(
+            p.entities.begin(),
+            p.entities.end(),
+            [&e](const std::reference_wrapper<Entity<TSeq>> & entity_ref) {
+                return entity_ref.get().get_id() == e.get_id();
+            }
+        ),
+        p.entities.end()
+    );
 
-    if (--p->n_entities > 0)
-    {
-
-        // When we move the end entity to the new location, the 
-        // moved entity needs to reflect the change, i.e., where the
-        // entity will now be located in the agent
-        size_t agent_location_in_last_entity  =
-            p->entities_locations[p->n_entities];
-
-        Entity<TSeq> * last_entity =
-            &m->get_entity(p->entities[p->n_entities]); ///< Last entity of the agent
-
-        // The end entity will be located where the removed was
-        last_entity->agents_location[agent_location_in_last_entity] =
-            idx_entity_in_agent;
-
-        // We now make the swap
-        std::swap(
-            p->entities[p->n_entities],
-            p->entities[idx_entity_in_agent]
-        );
-
-    }
-
-    if (--e->n_agents > 0)
-    {
-
-        // When we move the end agent to the new location, the 
-        // moved agent needs to reflect the change, i.e., where the
-        // agent will now be located in the entity
-        size_t entity_location_in_last_agent = e->agents_location[e->n_agents];
-        
-        Agent<TSeq> * last_agent  =
-            &m->get_agents()[e->agents[e->n_agents]]; ///< Last agent of the entity
-
-        // The end entity will be located where the removed was
-        last_agent->entities_locations[entity_location_in_last_agent] =
-            idx_agent_in_entity;
-
-        // We now make the swap
-        std::swap(
-            e->agents[e->n_agents],
-            e->agents[idx_agent_in_entity]
-        );
-
-    }
-
-    // Setting the date of the last removal
-    // e->date_last_add_or_remove = m->today();
+    // Remove agent from entity's agent list
+    e.agents.erase(
+        std::remove_if(
+            e.agents.begin(),
+            e.agents.end(),
+            [&p](const std::reference_wrapper<Agent<TSeq>> & agent_ref) {
+                return agent_ref.get().get_id() == p.get_id();
+            }
+        ),
+        e.agents.end()
+    );
 
     return;
 
